# -*- coding: utf-8 -*-
class ApplicationController < ActionController::Base
  protect_from_forgery

  # Authenticitaion CAS
  before_filter :casconfiguration
  before_filter CASClient::Frameworks::Rails::Filter 
  before_filter :checkCasClient   , :only  => [:accueil]

  # Vérification d'identité
  before_filter :checkGuestProfile , :only  => [:accueil,:data]
  before_filter :getuser,  :setConnectedUser     , :except  => [:login,:connect]

  # Construction du menu
  before_filter :buildNavigation  , :buildRubriques    , 
                :only  => [:composantes,:historiques,:journal,:show,:manage,:create,:configuration,
                           :index,:edit, :accueil,:filtres,:conf,:vues,:scripts,:default]

  # Filtres et rapports
  before_filter :mesfiltres , :only => [:index, :accueil,:show,:filtres,:conf,:sidebar]
  before_filter :mesfiltresPlus , :only => [:outils]
  before_filter :mesrapports , :only => [:index, :accueil,:show,:copy, :manage, :delete,:sidebar,:rendermesrapports]

  
  def setConnectedUser
    if session[:raw_userid] != nil
      @connectedUser=User.find(session[:raw_userid])
      User.connected= @connectedUser
    else
      @connectedUser=nil
      User.connected=nil
    end
  end

  def casconfiguration
    CASClient::Frameworks::Rails::Filter.configure(:cas_base_url => Appconfig.CAS_URL)
  end
  
  def checkCasClient
     return true if !session[:cas]     # Authentification CAS non demandé ?
     return true if session[:userid]   # Une session existe déja, la construction des groupes
                                       # se fera à la prochaine connexion
     user=User.find_by_uid(session[:cas_user])
     
     # Aucun utilisateur trouvé, dans la base et pas de création à la volée on quitte
     if user==nil && !Appconfig.CAS_USER_ONTHEFLY
       @username=session[:cas_extra_attributes][:displayName]
       usernotfound 
       return false
     end

     # Initialsisation du current_user
     User.current=nil

     # Aucun utilisateur, mais créaton à la volée est activée
     if user==nil && Appconfig.CAS_USER_ONTHEFLY
       mandatoryValue=session[:cas_extra_attributes][Appconfig.CAS_ATTRIBUTE_MANDATORY_NAME]
       # Est-ce que la valeur de l'attribut CAS MANDATORY est égal à celui de la configuration ?
       if mandatoryValue==Appconfig.CAS_ATTRIBUTE_MANDATORY_VALUE
         user=User.create(:nom    => session[:cas_extra_attributes][Appconfig.CAS_ATTRIBUTE_NAME],
                          :prenom => session[:cas_extra_attributes][Appconfig.CAS_ATTRIBUTE_FIRSTNAME],
                          :uid    => session[:cas_user])
         # On lui affecte le profil de All par défaut
         user.profile=Group.all.profile.dup
         user.profile.filtres=Group.all.profile.filtres                 
         user.profile.save                 
       # Sinon, (attribut obligatoire non trouvé) on invalide la session
       else
         usernotfound
         return false
       end

     end

     # On enlève les groupes dynamiques
     user.groups=user.groups.delete_if  {|g| g.dynamic }
     user.save

     # OK utilisateur trouvé, on initialise la session
     user.groups << Group.cas_user   # Appartient au groupe CAS_USER

     # Affectation des groupes dynamiques
     conf=Appconfig.CAS_ATTRIBUTE_GROUPE
     if conf != nil
       #logger.info("===== AFFECTATION DE GROUPE DYNAMIC ==============")
       #logger.info("#{conf}")
       conf.split(",").each do |g|
         Group.dynamique(session[:cas_extra_attributes][g],user,"SSO")
       end
       user.save
       #logger.info("=============================================")
     end
     
     # Ajout des groupes Zephir
     user.addToGroupesFromzephir()

     InitialiserLaSessionPour(user)

     
  end
  
 
  def mesfiltres
    @filtres=Filtre.currents
    @filtres.sort! { |a,b| a.groupe <=> b.groupe }
  end
  
  def mesfiltresPlus
     @filtres=Filtre.currentsFiltreVue
     @filtres.sort! { |a,b| a.groupe <=> b.groupe } 
     
     filtres=Filtre.currents
     filtres.sort! { |a,b| a.groupe <=> b.groupe }
     @filtres.concat(filtres)
  end

  def mesrapports
    @rapports=Rapport.currents
  end
  
  def unset_current_entity
    User.current.unsetCurrentEntity
  end
  
  def require_auth
    redirect_to :root if User.current.guest?   
  end
  
  def require_manager
    redirect_to :root if ! User.current.manager?   
  end

  def require_admin
    redirect_to :root if ! User.current.admin?
  end
  
  def flasher(action="")
    respond_to do |format|
        format.js {
          render :partial => "shared/flash" ,  :locals => { :flash => flash, :action => action }
       }
      end    
  end
  
  def unauthorized
    logger.info("=== UNAUTHORIZED ====")
    respond_to do |format|
        flash.now[:error]="Vous n'êtes pas autorisé à effectuer cette action"
        format.html {render :template => "shared/unauthorized", :layout=> false}
        format.js {
          render :partial => "shared/flash" ,  :locals => { :flash => flash }
       }
      end  
  end
  
  def usernotfound
    respond_to do |format|
        format.html {render :template => "shared/usernotfound", :layout=> false}
      end  
  end

  def usernotactif
    respond_to do |format|
      format.html {render :template => "shared/usernotactif", :layout=> false}
    end
  end
  
  def erroronsave
    respond_to do |format|
        flash.now[:error]="Une erreur est survenue lors de l'enregistrement"
        format.html {render :template => "shared/unauthorized", :layout=> false}
        format.js {
          render :partial => "shared/flash" ,  :locals => { :flash => flash }
       }
      end  
  end

  def readable?(objet)
    if ! objet.readable?
      unauthorized; return false
    end
    true
  end
  
  def isReadable?(objet)
    if ! objet.isReadable?
      unauthorized; return false    
    end  
    true
  end
  
  def writable?(objet)
    if ! objet.writable?
      unauthorized; return false    
    end  
    true
  end
  
  def saveObject(objet)
    if !objet.save
       erroronsave; return false  
    end
    true
  end
  
  def checkGuestProfile

    #logger.info("USER=#{session[:userid]} guest?=#{session[:guest]}")
    # Mode invité ?
    if params[:profile] && Appconfig.GEN_ALLOW_GUEST
      logger.info("Profile trouvé=#{params[:profile]}")
      g=Group.find_by_uid(params[:profile])
      if g!=nil && g.guest?
        user=User.create(:nom => "Invité", :prenom => params[:profile], :uid => "--"+Time.now.to_s())
        user.groups=[Group.guest,Group.all,g]
        user.profile=g.profile.dup
        user.profile.filtres=g.profile.filtres
        user.profile.save
        user.guest=true
        user.save
        session[:userid]=user.id
        session[:raw_userid]=user.id
        session[:guest]=true 
        User.current=user     
      else
         unauthorized; return false
      end 
    #elsif session[:guest]
    #  reset_session
    #  redirect_to  :root     
    end
    
  end


  def getuser

    # L'utilistaeur est déja reconnu
    if  session[:userid] != nil
      User.current= User.find(session[:userid])
      User.current.update_attribute(:lastaction,Time.now)
      # TODO: a voir si ça marche
      session[:expires] = 30.minutes.from_now
    # Pas encore reconnu et le mode invité est actif
    elsif Appconfig.GEN_ALLOW_GUEST
      u=User.create(:nom => "", :prenom => "", :uid => "--"+Time.now.to_s())
      u.groups=[Group.guest,Group.all]
      u.profile=Group.all.profile.dup
      u.profile.filtres=Group.all.profile.filtres
      u.profile.vues_order=Group.all.profile.vues_order
      u.profile.vues_active=Group.all.profile.vues_active
      u.profile.save
      u.guest=true
      u.save
      session[:userid]=u.id
      session[:raw_userid]=u.id
      User.current=u
    else
      redirect_to  :controller => "session" , :action => "login"
      return false
    end

    if User.current && ! User.current.actif?
      usernotactif
      return false
    end

    true
  end

  def buildRubriques

  end

  def menu_for_user(user)
    a=[]
    user.menus.each do |menu|
      next if ! menu.main?
      a<< menu
    end
    a
  end

  def  buildNavigation
    if session[:menu] 
      @menu_items=session[:menu] 
      @current_menu='accueil'
      return
    end
    @menu_items=[]

    @menu_items.concat menu_for_user(User.current)
    User.current.groups.each do |group|
      @menu_items.concat menu_for_user(group)
    end

    @menu_items=@menu_items.uniq{|x| x}
    @current_menu='accueil'
    session[:menu] = @menu_items
  end

  def unset_menu
    @current_menu=""
    @current_rubrique=""
    
  end

  def set_menu(opts={})
    
    @current_menu=opts[:menu]
    @current_rubrique=opts[:rubrique]
    @menu=Menu.find_by_tag(@current_menu)
    @current_entity=""
    #redirect_to controller: :main if @menu_items.index(@menu)  == nil
    
  end
  
  def randomId(size=16)
    rand(36**size).to_s(36)
  end
  
  def InitialiserLaSessionPour(user)
     # Récupération des informations su Zephir
     user.addToGroupesFromzephir()
    
     session[:userid]=user.id
     session[:raw_userid]=session[:userid]
     session[:guest]=false
     User.current= User.find(session[:userid])  
  end

end
