class Objet < ActiveRecord::Base

  attr_accessible :nom , :description , :data , :categorie
  
  serialize :data

  RAPPORT="RAPPORT"
  FILTRE="FILTRE"
  FILTRE_VUE="FILTRE_VUE"
  VUE="VUE"
  PERSPECTIVE="PERSPECTIVE"
  SCRIPT="SCRIPT"

  has_many   :rights
  belongs_to :user   , :foreign_key => "owner_id"

  before_save do
    #L'utilisateur a t'il le droit pour modifier
    if !aLeDroitDe Permission::WRITE
      errors.new[:base] << "Vous n'avez pas la permission de modifier cet objet"
      false
    end
  end
  
  before_destroy do
    if filtreVue?
      sql = "DELETE FROM filtres_vues WHERE filtre_id=#{id}"
      logger.info("===== BEFORE DESTROY==== #{sql}")
      ActiveRecord::Base.connection.execute(sql)  
    end
  end

  def initialize(*params)
    super(*params)
    self.owner=User.current
    self.position=0
  end

  def filtre?
    categorie==FILTRE ||  categorie==FILTRE_VUE
  end
  
  def filtreVue?
    categorie==FILTRE_VUE
  end

  def perspective?
    categorie==PERSPECTIVE
  end
  
  def script?
    categorie==SCRIPT  
  end

  def addRight(user,droit)

    right=Right.new
    right.user=user ; right.objet=self
    right.droit=Permission::READ
    right.save
    self.rights<<right
    self.save

    #right=Right.new(droit: droit)
    #right.objet=self; right.user=user
    #right.save
    #rights<<right
    #save
  end

  def commun=(params={})
    self.nom=params[:nom]
    self.description=params[:description]
  end

  def self.currents(cat)
    Objet.all(:conditions => ["owner_id = ? AND categorie = ?",User.current.id,cat])
  end

  def writable?(useEdmitMode=true)
    return false if useEdmitMode && !User.current.has(Profile::EDIT_MODE) 
    aLeDroitDe Permission::WRITE
  end

  def deletable?
    aLeDroitDe Permission::DELETE
  end

  def readable?
    aLeDroitDe Permission::READ
  end

  def aLeDroitDe(droit)
    return true if self.user==User.current
    return true if User.current.groups.exists? user
    return false if !User.current.has(Profile::SHOW_SHARED)
    rights.each do |r|
      return true if r.droitForCurrentUser?(droit)
    end
    false
  end

  def owner=(o)
    self.user=o
  end

  def owner()
    self.user
  end
  
  def owner?
    user==User.current  
  end

  def self.filtrer(objets)
    objets.delete_if {|o| !o.readable? }
    #logger.info("OBJETS: #{objets}")
    #objets
  end
  
  def ui
    categorie.downcase+"s"
  end
  
  def uiDisplay
    nom
  end
  
  def cast
    Script.find(id) if categorie==SCRIPT
  end

end
