class User < ActiveRecord::Base
  attr_accessible :nom, :prenom, :uid , :password, :mail, :actif
  attr_accessible :entities  , :special  , :lastaction  , :categorie
  attr_accessible :dynamic
  attr_accessible  :zephir_uid, :zephir_profil,:zephir_password

  ANONYMOUS="anonymous"

  has_many :permissions, :dependent => :destroy
  has_many :composantes, :through => :permissions

  has_many :objets
  has_one  :right

  belongs_to :user
  belongs_to :profile
  belongs_to :filtre

  has_and_belongs_to_many :groups
  has_and_belongs_to_many :menus

  serialize :entities
  serialize :zephir_profil
  
  include Zephir

  USER="user"
  GROUP="group"

  before_create do
    self.categorie=USER
    self.actif=true

    # Tout le monde fait partie du groupe ALL
    if groups.index(Group.all) == nil  && ! group?
      groups<<Group.all
    end

    if profile == nil
      self.profile=Profile.create
    end

    self.guest=false
    self.user=User.current  if User.current !=nil
    self.user=Group.admin  if User.current ==nil
    self.entities={} if self.entities==nil
    self.zephir_profil=Zephir.defaultProfil() 

  end


  before_save do
    self.groups=self.groups.uniq
    self.zephir_uid=self.uid if  self.zephir_uid == nil
    true
  end

  after_find do
    self.entities={} if self.entities==nil
    @permissions_cache={}
    @entities=nil 
  end

  #def initialize(*params)
  #  super(*params)
  #  self.categorie=USER
  #end

  # ========================================================
  # Gestion du cache pour les rapports
  # ceci évite de récupérer les permissions dans la base
  # a chaque entités
  def addPermissionsCache(composante,perm)
    @permissions_cache[composante.id]=perm  
  end
  
  def getPermissionsCache(composante)
    @permissions_cache={} if @permissions_cache==nil
    @permissions_cache[composante.id]
  end
  
  def clearPermissionCache()
    @permissions_cache.clear 
  end
  # =======================================================

  # =======================================================
  # Gestion des dernieres entités pour l'utilsateur
  # ie : Ceux visible dans la barre de navigation (5 maximum)
  def showEntity(uid,action=nil)
    entities[uid]=action #if ! entities.include? uid
    entities.shift if  entities.size>20
    @current_entity=uid
    update_attributes({entities: entities})
    uid
  end
  
  def getEntityAction(uid)
    entities[uid]
  end
  
  def remove_entity(uid)
    entities.delete(uid)  
    update_attributes({entities: entities})
  end
  
  def current_entity
    @current_entity  
  end
  
  def current_entity=uid
    @current_entity=uid  
  end
  
  def unsetCurrentEntity
    @current_entity = "" 
  end
  
  def save_entity_action(entity,action)
    
  end
  # =========================================================

  def entitiesFilter
      arr=getZephirEntities()
      arr=User.find(user_id).entitiesFilter if user_id!=nil   # On récupère les filtres du parent
      arr=arr & filtre.evaluation.content if filtre != nil && !arr.empty?  # Filtres définis pour l'utilsateur par son manager

      groups.each do |g|
        next if g.filtre==nil
        c=g.filtre.evaluation.content
        next if c.empty?
        arr=c if arr.empty?
        arr=arr | c if g.user_id==user_id  #
        arr=arr & c if g.user_id!=user_id && ! arr.empty?#
        arr= c      if g.user_id!=user_id &&  arr.empty?#
      end

      # Aucun établissement, on utilise le filtre par défaut
      if arr.empty? && !Appconfig.GEN_GUEST_FILTER.empty?
        logger.info("FILTRE PAR DEFAUT")
        f=Filtre.find_by_id(Appconfig.GEN_GUEST_FILTER)
        arr= f.evaluation.content if f !=nil
        if f == nil
          Entities.getAll().each do |value|
            arr<<value.valeur
          end
        end
      end
      
      # Aucun établissement et pas de filtre par défaut
      # On récupère tous les étabs
      if arr.empty? && Appconfig.GEN_GUEST_FILTER.empty?
        Entities.getAll().each do |value|
            arr<<value.valeur
          end  
      end

      #logger.info("=====> entitiesFilter #{arr}")

      arr
  end
  
  def has_perspective?
    !Perspective.currents.empty?
  end
  
  def hasEntity?(uid)
    @entities=entitiesFilter if @entities == nil
    @entities.include?  uid 
  end

  # =========================================================
  # Récupère les utilisateurs créés par l'utilisateur connecté
  def self.allManaged
    a=[User.connected.id]
    User.connected.groups.each do |g|
      a<<g.id
    end
    if User.connected.admin?
      a<<Group.guest.id
    end
    # Guest = 0 (Pas les utilisateurs créés en invité)
    User.find_all_by_guest(0,:conditions => ["user_id in (?)  ",a])
  end

  # managed? return true si c'est l'utilisateur qui est le créateur du user
  # ou si c'est l'administrateur
  def managed?
    user_id==User.current.id || User.current.admin?
  end

  # Groupes disponibles pour l'utilisateur,
  # EN fonction de User.current
  def groupsAvailable

    arr=[]

    # Tous les groupes créer par User
    arr.concat(Group.find_all_by_user_id_and_categorie(User.current.id,GROUP))

    # Si Admin tous les groupes spéciaux
    if User.current.admin?
      arr.concat(Group.find_all_by_user_id_and_categorie(Group.admin.id,GROUP))
      arr.concat(Group.find_all_by_special(true))
    end

    # Groupes disponibles, il y a aussi les groupes non spéciaux de l'utilisateur courrant
    arr.concat(User.current.groups.select {|v| ! v.special?  } )

    arr.uniq!{|x| x.id}
  end


  def readable?
    User.current.memberOf? user
  end

  def match(critere)
    return true if uid.downcase.include?(critere)
    return true if !nom.empty? && nom.downcase.include?(critere)
    false
  end

  # Return true si  self==user  ou les groupes contiennent user
  def memberOf?(user)
    return true if id==user.id
    groups.exists? user
  end

  def group?
    uid.first=="@"
  end

  def guest?
    memberOf? Group.guest
  end
  
  def gps?
    memberOf? Group.gps
  end
  
  def connected?
    ! guest
  end

  def manager?
    (memberOf? Group.manager) || admin?
  end

  def admin?
    memberOf? Group.admin
  end
  
  def self.admin
    User.find_by_uid("admin")
  end



  def get(key)
    profile.get key
  end

  def set(key,value)
    profile.set(key,value)
  end

  def has(key)
    profile.has(key)
  end

  def self.anonymous
    User.find_by_uid(ANONYMOUS)
  end
  
  def filtreModeET
    return profile.get(Profile::FILTRE_MODE)=="0" || profile.get(Profile::FILTRE_MODE)==false
  end

  def filtreModeOU
    return !filtreModeET
  end
  
  def currentComposante
    profile.get(Profile::COMPOSANTE)
  end
  
  def currentComposante=(c)
    profile.set(Profile::COMPOSANTE,c)
    profile.save
  end

  
  def for_json()
      { :id=>id, :cell => [ id,ApplicationController.helpers.format_user_categorie(categorie).pluralize,
                            ApplicationController.helpers.format_user(self) ]   }
  end
  
  # ============================= GESTION DE ZEPHIR =========================
  def getZephirEntities()
    zephir_profil[:entities]
  end
  
  def getZephirGroupes()
    zephir_profil[:groupes]
  end
  
  # Ajout des groupes Zephir
  def addToGroupesFromzephir()
    zephir()
    groupes=getZephirGroupes()
    groupes.each do |g|
         Group.dynamique(g,self,"ZEPHIR")
     end 
     self.save 
  end
  
  def zephir
    if Appconfig.ZEPHIR
      self.zephir_profil= Zephir.getUserRights(self)
    else
      self.zephir_profil={groupes: [], entities: []}
    end
    #logger.info("========= Recup ZEPHIR ============")
    #logger.info("#{self.zephir_profil}")
    #logger.info("=======================================")
    
    self.save

  end
  # ==============================================================================

  def self.current
    Thread.current[:user]
  end
  def self.current=(user)
    Thread.current[:user] = user
  end

  def self.connected
    Thread.current[:raw_user]
  end

  def self.connected=(user)
    Thread.current[:raw_user] = user
  end

end
