module Entities

  @@entities={}

  class Entity

    attr_accessor :uid, :libelle,:lat, :lon

    def initialize(uid, libelle,lat,lon)
      @uid = uid
      @libelle=libelle
      @lon = lon
      @lat=lat
    end

    def to_s
      "#{libelle} (#{@uid})"
    end
    
    def empty?
      (@lon>=-1 && @lon<=1) || @libelle==nil || @libelle.empty?  || @libelle == Value::NOTEXIST
    end

  end

  MARKER_PATH="/assets/markers/"

  # Retourne tous les étalisstements
  def self.getAll()
    Value.all(:select => "valeur",:conditions => ["composante_id = ?",Master.composanteKey.id])
  end
  
  # Retourne que les établissements de User.curent
  def self.getCurrents()
    User.current.entitiesFilter
  end
  
  def self.clear(uid)
    @@entities.delete(uid)
    self.get(uid)
  end


  def self.get(uid)
    if ! @@entities.include? uid
      lat=Master.getLat uid
      lon=Master.getLon  uid
      libelle=Master.getLibelle  uid
      e=Entity.new(uid,libelle,lat,lon)
      @@entities[uid]=e
    else
      e=@@entities[uid]
    end
    e

  end
  


  
  
  def self.evaluerTousLesFiltres
    alls=Filtre.getAll
    alls.each do |f|
      puts "Calcule du filtre #{f.nom}"
      f.clear
      Entities.computeFilter(f)
      puts "Nb entities: #{f.evaluation.count}"
    end    
  end

  def self.computeFilter(filtre)
    #puts "Calcule du filtre #{filtre.nom}"
    values=Entities.getAll()
    values.each do |value|
      filtre.evaluer(value.valeur,true,false)
    end
    filtre.evaluation.compiled=true
    filtre.evaluation.save
     

  end
  
  # Retourne les markers au format json
  # Interpretables par mapstraction
  def self.getMarkers(logger)
    array=[]

    # Tous les filtres et vues
    filtres=User.current.profile.getFiltres
    vues=Vue.currents
    
    #logger.info("#{User.current.profile.id}  #{filtres}")

    values=self.getCurrents()
    values.each do |uid|
      #value.valeur contient les uid des entités
      # Pour chaque valeur on récupère les informations du Master

      #uid=value.valeur if value.is_a? Value

      e=Entities.get(uid)
      next if e.empty?


      # Vérifie la correspondance des filtres ==============
      b=filtres.empty?
      bModeOu=User.current.filtreModeOU
      filtres.each do |filtre|
        if filtre.evaluer(uid)
           b=true
           break if bModeOu
        elsif !bModeOu
          b=false
          break 
        end
      end
      next if !b # Filtre ne match pas on passe au suivant
      # ======================================================

      icones=[]
      stop=false
      uidicone="|"
      # Calcule les vues
      vues.each do |vue|
        next if !vue.actif?
        vue.filtres.each do |filtre|
          break if stop
          next  if ! filtre.selected?
          if filtre.evaluer(uid) || (filtre.pique? && icones.empty?) 
            icones<<filtre
            uidicone<< "#{filtre.id}|"
            if filtre.exclusif?   # Si il est exclusif, on ne regarde pas les autres
              stop=true
              break 
            end
          end
        end
      end

      if icones.empty?
        icone="#{MARKER_PATH}/default.png"
        next
      elsif icones.size==1
        icone="#{MARKER_PATH}/#{icones.first().pictogramme}"
      else
        ico=Icone.find_by_uid(uidicone)
        if ico==nil
          ico=Icone.createFromFilters(uidicone,icones)
        end
        icone=ico.path
      end
      
      array<< {type: "Feature",
             title: "#{uid}",
             description: " #{e.libelle}",
             id: uid,
             geometry: {type: "Point", coordinates: [e.lon, e.lat]},
             icon: icone,
             icon_size: [24,24],
             properties: {prop0: "value0",  infoBubble: "Contents of bubble",}
             } 
      
    end

    filtres.each do |filtre|
      filtre.evaluation.compiled=true
      filtre.evaluation.save if filtre.evaluation.updated?
    end

    vues.each do |vue|
      vue.filtres.each do |filtre|
        next if ! filtre.selected?
        filtre.evaluation.compiled=true
        filtre.evaluation.save if filtre.evaluation.updated?
      end
    end
    
    array
  end


end