# -*- coding: utf-8 -*-

module Importations

  include Utils
  require 'open-uri'
  
  PATH="data/importations/"
  
  def self.perspective(filename)
    data = File.read(filename)
    Perspective.import(data)  
  end

   
  # Récupération des composantes dans une entête de fichier CSV
  # Retourne la composantekey, sa position et les composantes
  def getComposantes(columns,sep)
    puts "Recherche de composantes avec "
    puts " =>#{columns}"
    composantes=[]
    ckey=nil # La composante servant d'identifiant, on prend la première trouvée
    index_ckey=-1  # Indice
    a=columns.split(sep)
    (0..a.size-1).each do |i|
      #c=Composante.find_by_fullname(a[i])
      c=Composante.getComposanteFromFullname(a[i])
      composantes<<c
      if c==nil
        puts "[#{a[i]}] introuvable"
        next
      end
      
      ckey=c if c.flaguid? and ckey==nil
      index_ckey=i if c.flaguid? and index_ckey==-1
    end
    return ckey,index_ckey,composantes

  end


  # Parsing individuel : Utilisé par une action dont le script est "Par Serveur"
  # Cette méthode s'attend donc a voir en paramètre l'identifiant et le serveur
  # Les composantes peuvent être passées avec le format court, 
  # Si masteruid est null, la méthode va récupérer le masteruid dans Master.uid_<serveur>
  # Chaque ligne doit avoir le format  : <tag>;<valeur>
  def self.parseIndividual(identifiant,composante,input,serveur,masteruid=nil)

    source="<serveur>'#{serveur.composante.name}'</serveur>"
    count=0
    Value.transaction do
      ckey=composante.composanteKey
      lines=input.split("\n")
      lines.each do |line|
        line.strip!
        break if serveur.checkCanceled
        next if line.first == "#"
        values=line.split(";")
        next if values.count < 2

        # Est-ce que c'est une composante valide ?
        c=composante.getComposante(values[0])
        if c==nil
          c=Composante.getComposanteFromFullname(values[0])
        end
        next if c==nil
        
        valeur=values[1]
        count=count.next
        response=Value.setValue(ckey: ckey, ckeyvalue: identifiant,
                       composante: c, value: valeur, source:source,masteruid: masteruid)    
        if response.error?
          puts "\n\t\tECHEC: ckey: #{ckey}, ckeyvalue: #{identifiant},composante: #{c.fullname}, value:  #{valeur} => #{response.message}"
        end               
                                 
                       
      end
      # Moins de 5 lignes, peut etre une erreur
      if lines.count < 5 || count == 0
        puts"\n Output < 5 lignes"
        puts "=============="
        puts "#{input}"
        puts "==============="
      end
      puts "=> Nb:#{count}/#{lines.count}"
    end

  end


  def self.parseGlobal(serveur,input,serveurTask)
      print  "\tParsing global :"
      #ckey=serveur.composanteKey
      ckey=Master.serveur.composanteKey
      lines=input.split("\n")
      total=lines.count
      i=0
      source="<serveur>'#{serveurTask.composante.name}'</serveur>"
      print "---%"
      lines.each do |line|
        line.strip!
        i=i+1
        next if line.first == "#"
        values=line.split("|")
        entity=values.delete_at 0
        e=Entities.get(entity)
        #Value.setValue(ckey: ckey, ckeyvalue: uid,composante:)
        break if serveurTask.reload.checkCanceled
        serveurTask.update_attributes({:current => "#{e}",:percent => ((i*100)/total).to_i() })
        print "\b\b\b\b"
        print "#{(i*100/lines.count).to_s.rjust(3, ' ')}%"
        #puts("====================================")
        Value.transaction do
          values.each do |value|
            keyval= value.split("%%")
            tag=keyval[0]
            valeur=keyval[1]
            Value.setValue(ckey: ckey, ckeyvalue: entity,
                           composante: serveur.getComposante(tag), 
                           value: valeur,source: source)
          end
        end
        #puts("====================================")
    end
    print "\b\b\b\b"
    puts "100%"

  end
  
  def script(filename,nom,description,byserver=false,serveurs=[])
    script=Script.create
    script.nom=nom
    script.description=description
    script.byserver=byserver
    script.serveurs=serveurs
    
    File.open(filename, "r") do |aFile|
      lines=aFile.readlines
      script.data.script= lines.join("") 
      script.data.findLanguage(lines[0]) 
    end
    script.save
    script
  end
  
  def sentinellev2_subsystem(element,parent)
    

    #puts "Importation d'un subsystem dans #{parent.fullname}"
    
    return if ! parent.isWritable? 
    
    libelle=element.attributes["libelle"]
    icon=element.attributes["icon"]
    tag=element.attributes["type"]
    
    return if tag=="ACTION"
    
    if parent.children.exists? :tag => tag
        folder=parent.getComposante( tag )
    else
        folder=Composante.create(:tag => tag, 
                                 :name => libelle,:parent_id => parent.id, 
                                 :categorie => Composante::FOLDER)
    end
    
    element.elements.each("subsystem") do  |element| 
      sentinellev2_subsystem(element,folder)
    end
    
    element.elements.each("service") do |service|
      sentinellev2_service(service,folder)  
    end

    element.elements.each("include") do |include|
      file=include.attributes["file"]
      sentinellev2(PATH+file,folder) 
    end

  end
  
  def sentinellev2_service(element,parent)
  
    #puts "Importation d'un service dans #{parent.fullname}"
    return if ! parent.isWritable?   
    
    libelle=element.attributes["libelle"]
    tag=element.attributes["tag"]
    
    value=nil
    element.elements.each("value") do |v|
      value=v  
    end
    return if value==nil
    
    type=value.attributes["type"]
    
    if parent.children.exists? :tag => tag
          c=parent.getComposante( tag )
          c.name=libelle
          c.description=libelle
          c.categorie=type
          c.save
    else
          c=Composante.create(:tag => tag, 
                              :name => libelle,
                              :description =>libelle,
                              :categorie => type,
                              :parent_id => parent.id)
   end
    
  end
  
  
  def sentinellev2(filename,parent)
  
    if ! File.exist?(filename)
      puts " => Fichier #{filename} introuvable, on ne fait rien"  
      return 
    end
  
    puts "Importation dun modèle Sentinelle v2 : #{filename} dans #{parent.fullname}"
  
    if ! parent.isWritable? 
      puts "Composante non accessible en écriture"
      return
    end

    require "rexml/document"
    
    file = File.new( filename )
    doc = REXML::Document.new file
    
    
    doc.elements.each("subsystem") do  |element| 
      sentinellev2_subsystem(element,parent)
    end
    
    doc.elements.each("service") do |service|
      sentinellev2_service(service,parent)  
    end
    
    doc.elements.each("include") do |include|
      file=include.attributes["file"]
      sentinellev2(PATH+file,parent) 
    end
    
     doc.elements.each("composantes/subsystem") do  |element| 
      sentinellev2_subsystem(element,parent)
    end
    
    doc.elements.each("composantes/service") do |service|
      sentinellev2_service(service,parent)  
    end
    
    doc.elements.each("composantes/include") do |include|
      file=include.attributes["file"]
      sentinellev2(PATH+file,parent) 
    end
    
    
  end 
  
  
  def sentinelleValues(composante,url)
    composante.children.each do |c|
      Importations::sentinelleValues(c,url)  
    end
    if composante.isLeaf?
      puts " => #{composante.fullname}"
      source="#{url}?tag=#{composante.fullname}"
      url = URI.parse(source)  
      open(url) do |f|
        
        Value.transaction do
            f.each_line do |valeur|
              a=valeur.strip().split("|")  
              next if a.count != 3
              #puts valeur
              t=a[0];u=a[1];v=a[2]
              Value.setComposanteValue(uid: u,
                                       composante: composante,
                                       valeur: v, source: "SentinelleV2")
           end
        end 
         
      end  
    end
  end
  
  
  def sentinelleHistory(entity,composante,url)
    if composante==nil
      puts "Composante non trouve"
      return
    end
    
    # Récupération de uid du serveur en fonction de son TAG
    cuid=Master.serveurUidsForComposante(composante)
    vuid=Value.getValue(entity,cuid)
    if vuid!=nil
      uid=vuid.toutesLesValeurs().first  # On prende le premier
    else
      puts("Aucun serveur trouvé pour la composante #{composante.fullname}")
      return
    end

    

    # Ouverture de l'URL
    source="#{url}?tags=#{entity}@#{composante.fullname}&end=-1&start=-1"
    
    puts "#{source}"
    
    require 'open-uri'
    url = URI.parse(source)
    count=0
    # On y va
    open(url) do |f|
      
      Value.transaction do
      
        f.each_line do |valeur|
          count=count+1
          a=valeur.strip().split("|")
          t=a[0]
          v=a[1]
          vh=ValueHistory.new(uid: uid, masteruid: entity,etat:Value::OK,nberreurs:0,
                              source: "SentinelleV2",
                              message:"",
                              valeur:v)
                              
          vh.composante=composante
          vh.created_at=Time.at(a[0].to_i/1000)
          vh.save
         end
         
      end
      
      puts " => #{count} valeurs importées"
      
      #puts "response: #{response.inspect}"
    end
  end
  
  
  def dicos(dir,parent)
    
    if ! File.directory?(dir)
      puts " => #{dir} n'est pas un dossier, on ne fait rien"  
      return  
    end
    
    puts "Importation du dossier de dicos : #{dir} dans #{parent.fullname}" 
    
    @files = Dir.glob("#{dir}/*.xml").sort_by {|s| s}
    for file in @files
      dico(file,parent)
    end
    
    
  end
  
  
  def dico(filename,parent)
    
    if ! File.exist?(filename)
      puts " => Fichier #{filename} introuvable, on ne fait rien"  
      return 
    end
    
    #puts "Importation du dico : #{filename} dans #{parent.fullname}" 
    
    if ! parent.isWritable? 
      puts "Composante non accessible en écriture"
      return
    end
    
    require "rexml/document"
    
    file = File.new( filename )
    doc = REXML::Document.new file
    doc.elements.each("creole/variables/family") do  |element| 
      
      # Si caché on ne les intègre pas
      # next if element.attributes["hidden"]=='True'
       
      expert=element.attributes["mode"] == "expert"
      
      familly=element.attributes["name"]
      if parent.children.exists? :tag => familly
        folder=parent.getComposante( familly )
      else
        folder=Composante.create(:tag => familly,:expert => expert, 
                                 :name => familly,:parent_id => parent.id, 
                                 :categorie => Composante::FOLDER)
      end
      
      element.elements.each("variable") do |variable|
        # Si caché on ne les intègre pas
        next if variable.attributes["hidden"]=='True' 
        
        tag="conf_"+variable.attributes["name"]
        expert=variable.attributes["mode"] == "expert"
        
        type=variable.attributes["type"]
        if parent.children.exists? :tag => tag
          c=parent.getComposante( tag )
          c.name=tag
          c.description=variable.attributes["description"]
          c.expert=expert
          c.categorie=type
          c.save
        else
          c=Composante.create(:tag => tag, 
                              :name => tag,
                              :description => variable.attributes["description"],
                              :expert => expert,:categorie => type,
                              :parent_id => folder.id)
        end
        
      end
      
      
    end
      
   #User.current=Group.admin;include Importations ; Importations::dico("data/importations/30_amon.xml",Composante.find_by_tag("infosetabs"))
    
    
    
    
  end


  def csv (file,sep=";",columns="")
    
    Value.transaction do

    puts "Importation de #{file}"

    ckey,index_ckey,composantes=getComposantes(columns,",")  if  columns!=""

      @response=Utils::Response.new

        File.open(file, "r") do |aFile|

          ckey,index_ckey,composantes=getComposantes(aFile.readline.strip!,sep)  if  composantes==nil
          if ckey==nil
            @response.error "Pas de Composante principale trouvée"
            break
          end

          source="Import CSV de <user>#{User.current.uid}</user>"

          ligne=1
          # Chaque ligne
          aFile.each_line do |line|
            line.strip!
            ligne=ligne+1
            values=line.split(sep)

            (0..values.size-1).each do |i|
              #next if index_ckey==i
              
              @response=Value.setValue(ckey: ckey, ckeyvalue: values[index_ckey], composante: composantes[i], value: values[i],source: source)
              
              if  @response==nil || @response.error? 
                m=@response.message
                @response.message="Ligne #{ligne} (#{line})"
                #flash.now[:error]="Erreur lors de l'importation"
                break
              end
            end # End each

            break if  @response.error?

          end # End each line
        end # End File.open

    end

    @response


  end



end
